@@ -1,6 +1,7 @@
  /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005
+                  Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +15,24 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
+/** \file disk.c */
+
+/**
+ * \addtogroup PedDisk
+ *
+ * \brief Disk label access.
+ * 
+ * Most programs will need to use ped_disk_new() or ped_disk_new_fresh() to get
+ * anything done.  A PedDisk is always associated with a device and has a
+ * partition table.  There are different types of partition tables (or disk
+ * labels).  These are represented by the PedDiskType enumeration.
+ *
+ * @{
+ */
+
 #include "config.h"
 
 #include <parted/parted.h>
@@ -76,6 +92,12 @@ void ped_unregister_disk_type (PedDiskType* type)
 		disk_types = type->next;
 }
 
+/**
+ * Return the next disk type registers, after "type".  If "type" is
+ * NULL, returns the first disk type.
+ *
+ * \return Next disk; NULL if "type" is the last registered disk type.
+ */
 PedDiskType*
 ped_disk_type_get_next (PedDiskType* type)
 {
@@ -85,6 +107,11 @@ ped_disk_type_get_next (PedDiskType* type)
 		return disk_types;
 }
 
+/**
+ * Return the disk type with a name of "name".
+ *
+ * \return Disk type; NULL if no match.
+ */
 PedDiskType*
 ped_disk_type_get (const char* name)
 {
@@ -100,6 +127,11 @@ ped_disk_type_get (const char* name)
 	return walk;
 }
 
+/**
+ * Return the type of partition table detected on "dev".
+ *
+ * \return Type; NULL if none was detected.
+ */
 PedDiskType*
 ped_disk_probe (PedDevice* dev)
 {
@@ -124,6 +156,16 @@ ped_disk_probe (PedDevice* dev)
 	return walk;
 }
 
+/**
+ * Read the partition table off a device (if one is found). 
+ * 
+ * \warning May modify \p dev->cylinders, \p dev->heads and \p dev->sectors
+ *      if the partition table indicates that the existing values
+ *      are incorrect.
+ * 
+ * \return A new \link _PedDisk PedDisk \endlink object;
+ *         NULL on failure (e.g. partition table not detected).
+ */
 PedDisk*
 ped_disk_new (PedDevice* dev)
 {
@@ -186,6 +228,11 @@ error:
 	return 0;
 }
 
+/**
+ * Clone a \link _PedDisk PedDisk \endlink object.
+ *
+ * \return Deep copy of \p old_disk, NULL on failure.
+ */
 PedDisk*
 ped_disk_duplicate (const PedDisk* old_disk)
 {
@@ -219,6 +266,14 @@ error:
 	return NULL;
 }
 
+/**
+ * Remove all identifying signatures of a partition table,
+ * except for partition tables of a given type.
+ * 
+ * \return 0 on error, 1 otherwise.
+ * 
+ * \sa ped_disk_clobber()
+ */
 int
 ped_disk_clobber_exclude (PedDevice* dev, const PedDiskType* exclude)
 {
@@ -256,12 +311,28 @@ error:
 	return 0;
 }
 
+/** 
+ * Remove all identifying signatures of a partition table,
+ * 
+ * \return 0 on error, 1 otherwise.
+ * 
+ * \sa ped_disk_clobber_exclude()
+ */
 int
 ped_disk_clobber (PedDevice* dev)
 {
 	return ped_disk_clobber_exclude (dev, NULL);
 }
 
+/**
+ * Create a new partition table on \p dev.
+ *
+ * This new partition table is only created in-memory, and nothing is written
+ * to disk until ped_disk_commit_to_dev() is called.
+ * 
+ * \return The newly constructed \link _PedDisk PedDisk \endlink,
+ *      NULL on failure.
+ */
 PedDisk*
 ped_disk_new_fresh (PedDevice* dev, const PedDiskType* type)
 {
@@ -287,7 +358,7 @@ error:
 }
 
 PedDisk*
-_ped_disk_alloc (PedDevice* dev, const PedDiskType* disk_type)
+_ped_disk_alloc (const PedDevice* dev, const PedDiskType* disk_type)
 {
 	PedDisk*	disk;
 
@@ -295,7 +366,7 @@ _ped_disk_alloc (PedDevice* dev, const PedDiskType* disk_type)
 	if (!disk)
 		goto error;
 
-	disk->dev = dev;
+	disk->dev = (PedDevice*)dev;
 	disk->type = disk_type;
 	disk->update_mode = 1;
 	disk->part_list = NULL;
@@ -315,6 +386,13 @@ _ped_disk_free (PedDisk* disk)
 	ped_free (disk);
 }
 
+/**
+ * Close \p disk.
+ *
+ * What this function does depends on the PedDiskType of \p disk,
+ * but you can generally assume that outstanding writes are flushed
+ * (this mainly means that _ped_disk_free is called).
+ */
 void
 ped_disk_destroy (PedDisk* disk)
 {
@@ -324,6 +402,19 @@ ped_disk_destroy (PedDisk* disk)
 	disk->type->ops->free (disk);
 }
 
+/**
+ * Tell the operating system kernel about the partition table layout
+ * of \p disk.
+ *
+ * This is rather loosely defined: for example, on old versions of Linux,
+ * it simply calls the BLKRRPART ioctl, which tells the kernel to
+ * reread the partition table. On newer versions (2.4.x), it will
+ * use the new blkpg interface to tell Linux where each partition
+ * starts/ends, etc. In this case, Linux does not need to have support for
+ * a specific type of partition table.
+ * 
+ * \return 0 on failure, 1 otherwise.
+ */
 int
 ped_disk_commit_to_os (PedDisk* disk)
 {
@@ -342,6 +433,12 @@ error:
 	return 0;
 }
 
+/**
+ * Write the changes made to the in-memory description
+ * of a partition table to the device.
+ *
+ * \return 0 on failure, 1 otherwise.
+ */
 int
 ped_disk_commit_to_dev (PedDisk* disk)
 {
@@ -377,6 +474,15 @@ error:
 	return 0;
 }
 
+/*
+ * This function writes the in-memory changes to a partition table to
+ * disk and informs the operating system of the changes.
+ *
+ * \note Equivalent to calling first ped_disk_commit_to_dev(), then
+ *      ped_disk_commit_to_os().
+ *
+ * \return 0 on failure, 1 otherwise.
+ */
 int
 ped_disk_commit (PedDisk* disk)
 {
@@ -385,6 +491,20 @@ ped_disk_commit (PedDisk* disk)
 	return ped_disk_commit_to_os (disk);
 }
 
+/**
+ * \addtogroup PedPartition
+ *
+ * @{
+ */
+
+/**
+ * Check whether a partition is mounted or busy in some
+ * other way.
+ *
+ * \note An extended partition is busy if any logical partitions are mounted.
+ *
+ * \return \c 1 if busy.
+ */
 int
 ped_partition_is_busy (const PedPartition* part)
 {
@@ -393,6 +513,10 @@ ped_partition_is_busy (const PedPartition* part)
 	return ped_architecture->disk_ops->partition_is_busy (part);
 }
 
+/**
+ * Return a path that can be used to address the partition in the
+ * operating system.
+ */
 char*
 ped_partition_get_path (const PedPartition* part)
 {
@@ -401,6 +525,25 @@ ped_partition_get_path (const PedPartition* part)
 	return ped_architecture->disk_ops->partition_get_path (part);
 }
 
+/** @} */
+
+/**
+ * \addtogroup PedDisk
+ *
+ * @{
+ */
+
+/**
+ * Perform a sanity check on a partition table.
+ *
+ * \note The check performed is generic (i.e. it does not depends on the label
+ *      type of the disk.
+ *
+ * \throws PED_EXCEPTION_WARNING if a partition type ID does not match the file
+ *      system on it.
+ *
+ * \return 0 if the check fails, 1 otherwise.
+ */
 int
 ped_disk_check (PedDisk* disk)
 {
@@ -426,15 +569,21 @@ ped_disk_check (PedDisk* disk)
 		max_length_error = PED_MAX (4096, walk->geom.length / 100);
 		if (!ped_geometry_test_inside (&walk->geom, geom)
 		    || length_error > max_length_error) {
-			if (ped_exception_throw (
+			char* part_size = ped_unit_format (disk->dev, walk->geom.length);
+			char* fs_size = ped_unit_format (disk->dev, geom->length);
+			PedExceptionOption choice;
+
+			choice = ped_exception_throw (
 				PED_EXCEPTION_WARNING,
 				PED_EXCEPTION_IGNORE_CANCEL,
-				_("Partition %d is %.3fMb, but the file "
-				  "system is %.3fMb."),
-				walk->num,
-				(float) walk->geom.length / 2 / 1024.0,
-				(float) geom->length / 2 / 1024.0)
-					!= PED_EXCEPTION_IGNORE)
+				_("Partition %d is %s, but the file system is "
+				  "%s."),
+				walk->num, part_size, fs_size);
+
+			ped_free (part_size);
+			ped_free (fs_size);
+
+			if (choice != PED_EXCEPTION_IGNORE)
 				return 0;
 		}
 	}
@@ -442,6 +591,12 @@ ped_disk_check (PedDisk* disk)
 	return 1;
 }
 
+/**
+ * This function checks if a particular type of partition table supports
+ * a feature.
+ *
+ * \return 1 if \p disk_type supports \p feature, 0 otherwise.
+ */
 int
 ped_disk_type_check_feature (const PedDiskType* disk_type,
 			     PedDiskTypeFeature feature)
@@ -449,6 +604,9 @@ ped_disk_type_check_feature (const PedDiskType* disk_type,
 	return (disk_type->features & feature) != 0;
 }
 
+/**
+ * Get the number of primary partitions.
+ */
 int
 ped_disk_get_primary_partition_count (PedDisk* disk)
 {
@@ -467,6 +625,9 @@ ped_disk_get_primary_partition_count (PedDisk* disk)
 	return count;
 }
 
+/**
+ * Get the highest partition number on \p disk.
+ */
 int
 ped_disk_get_last_partition_num (PedDisk* disk)
 {
@@ -484,6 +645,12 @@ ped_disk_get_last_partition_num (PedDisk* disk)
 	return highest;
 }
 
+/**
+ * Get the maximum number of (primary) partitions the disk label supports.
+ * 
+ * For example, MacIntosh partition maps can have different sizes,
+ * and accordingly support a different number of partitions.
+ */
 int
 ped_disk_get_max_primary_partition_count (const PedDisk* disk)
 {
@@ -494,7 +661,8 @@ ped_disk_get_max_primary_partition_count (const PedDisk* disk)
 	return disk->type->ops->get_max_primary_partition_count (disk);
 }
 
-/* We turned a really nasty beaurocracy problem into an elegant maths
+/**
+ * \internal We turned a really nasty bureaucracy problem into an elegant maths
  * problem :-)  Basically, there are some constraints to a partition's
  * geometry:
  *
@@ -544,7 +712,8 @@ _partition_enumerate (PedPartition* part)
 	return disk_type->ops->partition_enumerate (part);
 }
 
-/* gives all the (active) partitions a number.  It should preserve the numbers
+/**
+ * Gives all the (active) partitions a number.  It should preserve the numbers
  * and orders as much as possible.
  */
 static int
@@ -556,7 +725,7 @@ ped_disk_enumerate_partitions (PedDisk* disk)
 
 	PED_ASSERT (disk != NULL, return 0);
 
-/* first "sort" already-numbered partitions.  (Eg: if a logical partition
+/* first "sort" already-numbered partitions.  (e.g. if a logical partition
  * is removed, then all logical partitions that were number higher MUST be
  * renumbered)
  */
@@ -722,7 +891,8 @@ _disk_alloc_freespace (PedDisk* disk)
 	return 1;
 }
 
-/* update mode: used when updating the internal representation of the partition
+/**
+ * Update mode: used when updating the internal representation of the partition
  * table.  In update mode, the metadata and freespace placeholder/virtual
  * partitions are removed, making it much easier for various manipulation
  * routines...
@@ -772,6 +942,16 @@ _disk_pop_update_mode (PedDisk* disk)
 	}
 }
 
+/** @} */
+
+/**
+ * \addtogroup PedPartition
+ *
+ * \brief Partition access.
+ * 
+ * @{
+ */
+
 PedPartition*
 _ped_partition_alloc (const PedDisk* disk, PedPartitionType type,
 		      const PedFileSystemType* fs_type,
@@ -838,6 +1018,21 @@ fail:
 	return 0;
 }
 
+/**
+ * Create a new \link _PedPartition PedPartition \endlink on \p disk.
+ *
+ * \param type One of \p PED_PARTITION_PRIMARY, \p PED_PARTITION_EXTENDED,
+ *      \p PED_PARTITION_LOGICAL.
+ *
+ * \note The constructed partition is not added to <tt>disk</tt>'s
+ *      partition table. Use ped_disk_add_partition() to do this.
+ * 
+ * \return A new \link _PedPartition PedPartition \endlink object,
+ *      NULL on failure.
+ *
+ * \throws PED_EXCEPTION_ERROR if \p type is \p EXTENDED or \p LOGICAL but the
+ *      label does not support this concept.
+ */ 
 PedPartition*
 ped_partition_new (const PedDisk* disk, PedPartitionType type,
 		   const PedFileSystemType* fs_type, PedSector start,
@@ -880,6 +1075,12 @@ error:
 	return NULL;
 }
 
+/**
+ * Destroy a \link _PedPartition PedPartition \endlink object.
+ *
+ * \note Should not be called on a partition that is in a partition table.
+ *      Use ped_disk_delete_partition() instead.
+ */
 void
 ped_partition_destroy (PedPartition* part)
 {
@@ -890,6 +1091,13 @@ ped_partition_destroy (PedPartition* part)
 	part->disk->type->ops->partition_destroy (part);
 }
 
+
+/**
+ * Return whether or not the partition is "active".
+ *
+ * A partition is active if \p part->type is neither \p PED_PARTITION_METADATA
+ * nor \p PED_PARTITION_FREE.
+ */
 int
 ped_partition_is_active (const PedPartition* part)
 {
@@ -899,6 +1107,24 @@ ped_partition_is_active (const PedPartition* part)
 		 || part->type & PED_PARTITION_METADATA);
 }
 
+/**
+ * Set the state (\c 1 or \c 0) of a flag on a partition.
+ * 
+ * Flags are disk label specific, although they have a global
+ * "namespace": the flag PED_PARTITION_BOOT, for example, roughly means
+ * "this" partition is bootable". But this means different things on different
+ * disk labels (and may not be defined on some disk labels). For example,
+ * on MS-DOS disk labels, there can only be one boot partition, and this
+ * refers to the partition that will be booted from on startup. On PC98
+ * disk labels, the user can choose from any bootable partition on startup.
+ * 
+ * \note It is an error to call this on an unavailable flag -- use
+ * ped_partition_is_flag_available() to determine which flags are available
+ * for a given disk label.
+ *
+ * \throws PED_EXCEPTION_ERROR if the requested flag is not available for this
+ *      label.
+ */
 int
 ped_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 {
@@ -925,6 +1151,13 @@ ped_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 	return ops->partition_set_flag (part, flag, state);
 }
 
+/**
+ * Get the state (\c 1 or \c 0) of a flag on a partition.
+ *
+ * See ped_partition_set_flag() for conditions that must hold.
+ *
+ * \todo Where's the check for flag availability?
+ */
 int
 ped_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 {
@@ -937,6 +1170,11 @@ ped_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 	return part->disk->type->ops->partition_get_flag (part, flag);
 }
 
+/**
+ * Check whether a given flag is available on a partition.
+ *
+ * \return \c 1 if the flag is available.
+ */
 int
 ped_partition_is_flag_available (const PedPartition* part,
 	       			 PedPartitionFlag flag)
@@ -950,6 +1188,14 @@ ped_partition_is_flag_available (const PedPartition* part,
 	return part->disk->type->ops->partition_is_flag_available (part, flag);
 }
 
+/**
+ * Sets the system type on the partition to \p fs_type.
+ *
+ * \note The file system may be opened, to get more information about the
+ * file system, e.g. to determine if it's FAT16 or FAT32.
+ *
+ * \return \c 0 on failure.
+ */
 int
 ped_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 {
@@ -982,6 +1228,21 @@ _assert_partition_name_feature (const PedDiskType* disk_type)
 	return 1;
 }
 
+/**
+ * Sets the name of a partition.
+ *
+ * \note This will only work if the disk label supports it.
+ *      You can use
+ *      \code
+ * ped_disk_type_check_feature (part->disk->type, PED_DISK_TYPE_PARTITION_NAME);
+ *      \endcode
+ *      to check whether this feature is enabled for a label.
+ *      
+ * \note \p name will not be modified by libparted. It can be freed
+ *      by the caller immediately after ped_partition_set_name() is called.
+ *
+ * \return \c 1 on success, \c 0 otherwise.
+ */
 int
 ped_partition_set_name (PedPartition* part, const char* name)
 {
@@ -999,6 +1260,13 @@ ped_partition_set_name (PedPartition* part, const char* name)
 	return 1;
 }
 
+/**
+ * Returns the name of a partition \p part.  This will only work if the disk label
+ * supports it.
+ *
+ * \note The returned string should not be modified.  It should
+ *	not be referenced after the partition is destroyed.
+ */
 const char*
 ped_partition_get_name (const PedPartition* part)
 {
@@ -1014,6 +1282,14 @@ ped_partition_get_name (const PedPartition* part)
 	return part->disk->type->ops->partition_get_name (part);
 }
 
+/** @} */
+
+/**
+ * \addtogroup PedDisk
+ *
+ * @{
+ */
+
 PedPartition*
 ped_disk_extended_partition (const PedDisk* disk)
 {
@@ -1028,8 +1304,14 @@ ped_disk_extended_partition (const PedDisk* disk)
 	return walk;
 }
 
-/* returns the next partition.  If the current partition is an extended
- * partition, returns the first logical partition.
+/** 
+ * Return the next partition after \p part on \p disk. If \p part is \c NULL,
+ * return the first partition. If \p part is the last partition, returns
+ * \c NULL. If \p part is an extended partition, returns the first logical
+ * partition. If this is called repeatedly passing the return value as \p part,
+ * a depth-first traversal is executed.
+ *
+ * \return The next partition, \c NULL if no more partitions left.
  */
 PedPartition*
 ped_disk_next_partition (const PedDisk* disk, const PedPartition* part)
@@ -1047,6 +1329,8 @@ ped_disk_next_partition (const PedDisk* disk, const PedPartition* part)
 	return NULL;
 }
 
+/** @} */
+
 #ifdef DEBUG
 static int
 _disk_check_sanity (PedDisk* disk)
@@ -1073,6 +1357,11 @@ _disk_check_sanity (PedDisk* disk)
 }
 #endif
 
+/**
+ * Returns the partition numbered \p num. 
+ *
+ * \return \c NULL if the specified partition does not exist.
+ */
 PedPartition*
 ped_disk_get_partition (const PedDisk* disk, int num)
 {
@@ -1089,9 +1378,10 @@ ped_disk_get_partition (const PedDisk* disk, int num)
 	return NULL;
 }
 
-/* Returns the partition that contains sect.  If sect lies within a logical
+/**
+ * Returns the partition that contains sect.  If sect lies within a logical
  * partition, then the logical partition is returned (not the extended
- * partition)
+ * partition).
  */
 PedPartition*
 ped_disk_get_partition_by_sector (const PedDisk* disk, PedSector sect)
@@ -1177,7 +1467,8 @@ _disk_raw_remove (PedDisk* disk, PedPartition* part)
 	return 1;
 }
 
-/* UPDATE MODE ONLY
+/*
+ *UPDATE MODE ONLY
  */
 static int
 _disk_raw_add (PedDisk* disk, PedPartition* part)
@@ -1263,15 +1554,16 @@ _partition_get_overlap_constraint (PedPartition* part, PedGeometry* geom)
 	return ped_constraint_new_from_max (&free_space);
 }
 
-/* returns 0 if the partition, "part" overlaps with any partitions on the
- * "disk".  The geometry of "part" is taken to be "geom", NOT "part->geom"
- * (the idea here is to check if "geom" is valid, before changing "part").
+/*
+ * Returns \c 0 if the partition, \p part overlaps with any partitions on the
+ * \p disk.  The geometry of \p part is taken to be \p geom, NOT \p part->geom
+ * (the idea here is to check if \p geom is valid, before changing \p part).
  * 
  * This is useful for seeing if a resized partitions new geometry is going to
  * fit, without the existing geomtry getting in the way.
  *
  * Note: overlap with an extended partition is also allowed, provided that
- * "geom" lies completely inside the extended partition.
+ * \p geom lies completely inside the extended partition.
  */
 static int
 _disk_check_part_overlaps (PedDisk* disk, PedPartition* part)
@@ -1334,7 +1626,7 @@ _partition_check_basic_sanity (PedDisk* disk, PedPartition* part)
 			ped_exception_throw (
 				PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_CANCEL,
-				"Too many primary partitions.");
+				_("Too many primary partitions."));
 			return 0;
 		}
 	}
@@ -1367,7 +1659,7 @@ _check_extended_partition (PedDisk* disk, PedPartition* part)
 		ped_exception_throw (
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("Can't have more than one extended partition on %s"),
+			_("Can't have more than one extended partition on %s."),
 			disk->dev->path);
 		return 0;
 	}
@@ -1390,6 +1682,8 @@ _check_partition (PedDisk* disk, PedPartition* part)
 {
 	PedPartition*	ext_part = ped_disk_extended_partition (disk);
 
+	PED_ASSERT (part->geom.start <= part->geom.end, return 0);
+
 	if (part->type == PED_PARTITION_EXTENDED) {
 		if (!_check_extended_partition (disk, part))
 			return 0;
@@ -1410,7 +1704,7 @@ _check_partition (PedDisk* disk, PedPartition* part)
 		ped_exception_throw (
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("Can't have overlapping partitions"));
+			_("Can't have overlapping partitions."));
 		return 0;
 	}
 
@@ -1426,6 +1720,19 @@ _check_partition (PedDisk* disk, PedPartition* part)
 	return 1;
 }
 
+/**
+ * Adds PedPartition \p part to PedPartition \p disk.
+ * 
+ * \warning The partition's geometry may be changed, subject to \p constraint.
+ * You could set \p constraint to <tt>ped_constraint_exact(&part->geom)</tt>, but
+ * many partition table schemes have special requirements on the start
+ * and end of partitions.  Therefore, having an overly strict constraint
+ * will probably mean that this function will fail (in which
+ * case \p part will be left unmodified)
+ * \p part is assigned a number (\p part->num) in this process.
+ * 
+ * \return \c 0 on failure.
+ */
 int
 ped_disk_add_partition (PedDisk* disk, PedPartition* part,
 			const PedConstraint* constraint)
@@ -1447,6 +1754,14 @@ ped_disk_add_partition (PedDisk* disk, PedPartition* part,
 		constraints = ped_constraint_intersect (overlap_constraint,
 							constraint);
 
+		if (!constraints && constraint) {
+			ped_exception_throw (
+				PED_EXCEPTION_ERROR,
+				PED_EXCEPTION_CANCEL,
+				_("Can't have overlapping partitions."));
+			goto error;
+		}
+
 		if (!_partition_enumerate (part))
 			goto error;
 		if (!_partition_align (part, constraints))
@@ -1473,6 +1788,15 @@ error:
 	return 0;
 }
 
+/**
+ * Removes PedPartition \p part from PedDisk \p disk.
+ *
+ * If \p part is an extended partition, it must not contain any logical
+ * partitions. \p part is *NOT* destroyed. The caller must call
+ * ped_partition_destroy(), or use ped_disk_delete_partition() instead.
+ *
+ * \return \c 0 on error.
+ */
 int
 ped_disk_remove_partition (PedDisk* disk, PedPartition* part)
 {
@@ -1494,6 +1818,11 @@ error:
 static int
 ped_disk_delete_all_logical (PedDisk* disk);
 
+/**
+ * Removes \p part from \p disk, and destroys \p part.
+ *
+ * \return \c 0 on failure.
+ */
 int
 ped_disk_delete_partition (PedDisk* disk, PedPartition* part)
 {
@@ -1530,6 +1859,11 @@ ped_disk_delete_all_logical (PedDisk* disk)
 	return 1;
 }
 
+/**
+ * Removes and destroys all partitions on \p disk.
+ *
+ * \return \c 0 on failure.
+ */
 int
 ped_disk_delete_all (PedDisk* disk)
 {
@@ -1552,6 +1886,18 @@ ped_disk_delete_all (PedDisk* disk)
 	return 1;
 }
 
+/**
+ * Sets the geometry of \p part (i.e. change a partitions location). This can
+ * fail for many reasons, e.g. can't overlap with other partitions. If it
+ * does fail, \p part will remain unchanged. Returns \c 0 on failure. \p part's
+ * geometry may be set to something different from \p start and \p end subject
+ * to \p constraint.
+ *
+ * \warning The constraint warning from ped_disk_add_partition() applies.
+ * 
+ * \note this function does not modify the contents of the partition.  You need
+ *       to call ped_file_system_resize() separately.
+ */
 int
 ped_disk_set_partition_geom (PedDisk* disk, PedPartition* part,
 			     const PedConstraint* constraint,
@@ -1574,6 +1920,13 @@ ped_disk_set_partition_geom (PedDisk* disk, PedPartition* part,
 	overlap_constraint
 		= _partition_get_overlap_constraint (part, &new_geom);
 	constraints = ped_constraint_intersect (overlap_constraint, constraint);
+	if (!constraints && constraint) {
+		ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_CANCEL,
+			_("Can't have overlapping partitions."));
+		goto error_pop_update_mode;
+	}
 
 	part->geom = new_geom;
 	if (!_partition_align (part, constraints))
@@ -1600,6 +1953,12 @@ error:
 	return 0;
 }
 
+/**
+ * Grow PedPartition \p part geometry to the maximum possible subject to
+ * \p constraint.  The new geometry will be a superset of the old geometry.
+ * 
+ * \return 0 on failure
+ */
 int
 ped_disk_maximize_partition (PedDisk* disk, PedPartition* part,
 			     const PedConstraint* constraint)
@@ -1654,6 +2013,12 @@ error:
 	return 0;
 }
 
+/**
+ * Get the maximum geometry \p part can be grown to, subject to
+ * \p constraint.
+ *
+ * \return \c NULL on failure.
+ */
 PedGeometry*
 ped_disk_get_max_partition_geometry (PedDisk* disk, PedPartition* part,
 				     const PedConstraint* constraint)
@@ -1685,8 +2050,12 @@ ped_disk_get_max_partition_geometry (PedDisk* disk, PedPartition* part,
 	return max_geom;
 }
 
-/* Reduces the size of the extended partition to wrap the logical partitions.
- * If there are no logical partitions, it removes the extended partition.
+/** 
+ * Reduce the size of the extended partition to a minimum while still wrapping
+ * its logical partitions.  If there are no logical partitions, remove the
+ * extended partition.
+ *
+ * \return 0 on failure.
  */
 int
 ped_disk_minimize_extended_partition (PedDisk* disk)
@@ -1725,6 +2094,28 @@ ped_disk_minimize_extended_partition (PedDisk* disk)
 	return status;
 }
 
+/**
+ * @}
+ */
+
+/**
+ * \addtogroup PedPartition
+ *
+ * @{
+ */
+
+/**
+ * Returns a name that seems mildly appropriate for a partition type \p type.
+ * 
+ * Eg, if you pass (PED_PARTITION_LOGICAL & PED_PARTITION_FREESPACE), it
+ * will return "free".  This isn't to be taken too seriously - it's just
+ * useful for user interfaces, so you can show the user something ;-)
+ * 
+ * \note The returned string will be in English.  However,
+ * translations are provided, so the caller can call
+ * dgettext("parted", RESULT) on the result.
+ *
+ */
 const char*
 ped_partition_type_get_name (PedPartitionType type)
 {
@@ -1740,8 +2131,13 @@ ped_partition_type_get_name (PedPartitionType type)
 		return N_("primary");
 }
 
-/* returns the English name of the flag.  Get the native name by calling
- * dgettext("parted", XXX) on the English name.
+
+/**
+ * Returns a name for a \p flag, e.g. PED_PARTITION_BOOT will return "boot".
+ * 
+ * \note The returned string will be in English.  However,
+ * translations are provided, so the caller can call
+ * dgettext("parted", RESULT) on the result.
  */
 const char*
 ped_partition_flag_get_name (PedPartitionFlag flag)
@@ -1767,23 +2163,38 @@ ped_partition_flag_get_name (PedPartitionFlag flag)
 		return N_("palo");
 	case PED_PARTITION_PREP:
 		return N_("prep");
+	case PED_PARTITION_MSFT_RESERVED:
+		return N_("msftres");
 
 	default:
 		ped_exception_throw (
 			PED_EXCEPTION_BUG,
 			PED_EXCEPTION_CANCEL,
-			_("Unknown flag partition flag, %d."),
+			_("Unknown partition flag, %d."),
 			flag);
 		return NULL;
 	}
 }
 
+/**
+ * Iterates through all flags. 
+ * 
+ * ped_partition_flag_next(0) returns the first flag
+ *
+ * \return the next flag, or 0 if there are no more flags
+ */
 PedPartitionFlag
 ped_partition_flag_next (PedPartitionFlag flag)
 {
 	return (flag + 1) % (PED_PARTITION_LAST_FLAG + 1);
 }
 
+/**
+ * Returns the flag associated with \p name.  
+ *
+ * \p name can be the English
+ * string, or the translation for the native language.
+ */
 PedPartitionFlag
 ped_partition_flag_get_by_name (const char* name)
 {
@@ -1801,7 +2212,6 @@ ped_partition_flag_get_by_name (const char* name)
 	return 0;
 }
 
-
 void
 ped_partition_print (PedPartition* part)
 {
@@ -1813,6 +2223,17 @@ ped_partition_print (PedPartition* part)
 		(int) part->geom.start, (int) part->geom.end);
 }
 
+/** @} */
+
+/**
+ * \addtogroup PedDisk
+ *
+ * @{
+ */
+
+/**
+ * Prints a summary of disk's partitions.  Useful for debugging.
+ */
 void
 ped_disk_print (PedDisk* disk)
 {
@@ -1825,3 +2246,5 @@ ped_disk_print (PedDisk* disk)
 		ped_partition_print (part);
 }
 
+/** @} */
+
