@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -24,6 +24,8 @@
 
 #ifdef linux
 #  include <parted/linux.h>
+#elif defined(__BEOS__)
+#  include <parted/beos.h>
 #else
 #  include <parted/gnu.h>
 #endif
@@ -53,7 +55,7 @@ typedef struct
 } pointer_size_type;
 
 #ifdef DEBUG
-static pointer_size_type dodgey_malloc_list[] = {
+static pointer_size_type dodgy_malloc_list[] = {
  {0,		0},
  {0,		0},
  {0,		0},
@@ -66,7 +68,7 @@ static pointer_size_type dodgey_malloc_list[] = {
  {0,		0}
 };
 
-static int	dodgey_memory_active[100];
+static int	dodgy_memory_active[100];
 #endif /* DEBUG */
 
 int
@@ -79,10 +81,10 @@ ped_set_architecture (const PedArchitecture* arch)
 }
 
 extern void ped_disk_bsd_init ();
+extern void ped_disk_dvh_init ();
 extern void ped_disk_gpt_init ();
 extern void ped_disk_loop_init ();
 extern void ped_disk_mac_init ();
-extern void ped_disk_dvh_init ();
 extern void ped_disk_msdos_init ();
 extern void ped_disk_pc98_init ();
 extern void ped_disk_sun_init ();
@@ -93,16 +95,17 @@ init_disk_types ()
 {
 	ped_disk_loop_init ();	/* must be last in the probe list */
 
-	ped_disk_amiga_init ();
 	ped_disk_sun_init ();
 #ifdef ENABLE_PC98
 	ped_disk_pc98_init ();
 #endif
 	ped_disk_msdos_init ();
-	ped_disk_dvh_init ();
 	ped_disk_mac_init ();
 	ped_disk_gpt_init ();
+	ped_disk_dvh_init ();
 	ped_disk_bsd_init ();
+	ped_disk_amiga_init ();
+
 }
 
 #ifdef ENABLE_FS
@@ -134,10 +137,10 @@ init_file_system_types ()
 #endif /* ENABLE_FS */
 
 extern void ped_disk_bsd_done ();
+extern void ped_disk_dvh_done ();
 extern void ped_disk_gpt_done ();
 extern void ped_disk_loop_done ();
 extern void ped_disk_mac_done ();
-extern void ped_disk_dvh_done ();
 extern void ped_disk_msdos_done ();
 extern void ped_disk_pc98_done ();
 extern void ped_disk_sun_done ();
@@ -146,17 +149,17 @@ extern void ped_disk_amiga_done ();
 static void
 done_disk_types ()
 {
-	ped_disk_amiga_done ();
 	ped_disk_sun_done ();
 #ifdef ENABLE_PC98
 	ped_disk_pc98_done ();
 #endif
 	ped_disk_msdos_done ();
-	ped_disk_dvh_done ();
 	ped_disk_mac_done ();
 	ped_disk_loop_done ();
 	ped_disk_gpt_done ();
+	ped_disk_dvh_done ();
 	ped_disk_bsd_done ();
+	ped_disk_amiga_done ();
 }
 
 static void _init() __attribute__ ((constructor));
@@ -177,12 +180,14 @@ _init()
 	/* FIXME: a better way of doing this? */
 #ifdef linux
 	ped_set_architecture (&ped_linux_arch);
+#elif defined(__BEOS__)
+	ped_set_architecture (&ped_beos_arch);
 #else
 	ped_set_architecture (&ped_gnu_arch);
 #endif
 
 #ifdef DEBUG
-	memset (dodgey_memory_active, 0, sizeof (dodgey_memory_active));
+	memset (dodgy_memory_active, 0, sizeof (dodgy_memory_active));
 #endif
 }
 
@@ -236,16 +241,16 @@ ped_get_version ()
 
 #ifdef DEBUG
 static void
-_check_dodgey_pointer (const void* ptr, size_t size, int is_malloc)
+_check_dodgy_pointer (const void* ptr, size_t size, int is_malloc)
 {
 	int		i;
 
-	for (i=0; dodgey_malloc_list[i].pointer; i++) {
-		if (dodgey_malloc_list[i].pointer != ptr)
+	for (i=0; dodgy_malloc_list[i].pointer; i++) {
+		if (dodgy_malloc_list[i].pointer != ptr)
  			continue;
-		if (is_malloc && dodgey_malloc_list[i].size != size)
+		if (is_malloc && dodgy_malloc_list[i].size != size)
 			continue;
-		if (!is_malloc && !dodgey_memory_active[i])
+		if (!is_malloc && !dodgy_memory_active[i])
 			continue;
 
 
@@ -253,16 +258,16 @@ _check_dodgey_pointer (const void* ptr, size_t size, int is_malloc)
 			ped_exception_throw (
 				PED_EXCEPTION_INFORMATION,
 				PED_EXCEPTION_OK,
-				"Dodgey malloc(%x) == %p occured (active==%d)",
-				size, ptr, dodgey_memory_active[i]);
-			dodgey_memory_active[i]++;
+				"Dodgy malloc(%x) == %p occurred (active==%d)",
+				size, ptr, dodgy_memory_active[i]);
+			dodgy_memory_active[i]++;
 		} else {
 			ped_exception_throw (
 				PED_EXCEPTION_INFORMATION,
 				PED_EXCEPTION_OK,
-				"Dodgey free(%p) occured (active==%d)",
-				ptr, dodgey_memory_active[i]);
-			dodgey_memory_active[i]--;
+				"Dodgy free(%p) occurred (active==%d)",
+				ptr, dodgy_memory_active[i]);
+			dodgy_memory_active[i]--;
 		}
 
 		return;
@@ -284,7 +289,7 @@ ped_malloc (size_t size)
 
 #ifdef DEBUG
 	memset (mem, 0xff, size);
-	_check_dodgey_pointer (mem, size, 1);
+	_check_dodgy_pointer (mem, size, 1);
 #endif
 
 	return mem;
@@ -305,11 +310,22 @@ ped_realloc (void** old, size_t size)
 	return 1;
 }
 
+
+void* ped_calloc (size_t size)
+{
+        void* buf = ped_malloc (size);
+
+        memset (buf, 0, size);
+
+        return buf;
+}
+
+
 void
 ped_free (void* ptr)
 {
 #ifdef DEBUG
-	_check_dodgey_pointer (ptr, 0, 0);
+	_check_dodgy_pointer (ptr, 0, 0);
 #endif
 
 	free (ptr);
