@@ -1,6 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002, 2006 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include <parted/parted.h>
@@ -65,22 +65,29 @@ extern int tgetnum (char* key);
 char* prog_name = "GNU Parted " VERSION "\n";
 
 static char* banner_msg = N_(
-"Copyright (C) 1998 - 2005 Free Software Foundation, Inc.\n"
-"This program is free software, covered by the GNU General Public License.\n"
-"\n"
-"This program is distributed in the hope that it will be useful, "
-"but WITHOUT ANY WARRANTY; without even the implied warranty of "
-"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-"GNU General Public License for more details.\n\n");
+"Welcome to GNU Parted! Type 'help' to view a list of commands.\n");
 
 static char* usage_msg = N_(
 "Usage: parted [OPTION]... [DEVICE [COMMAND [PARAMETERS]...]...]\n"
-"Apply COMMANDs with PARAMETERS to DEVICE.  If no COMMAND(s) are given, runs in\n"
+"Apply COMMANDs with PARAMETERS to DEVICE.  If no COMMAND(s) are given, run in\n"
 "interactive mode.\n");
 
 static char* bug_msg = N_(
-"You found a bug in GNU Parted.  Please email a bug report to bug-parted@gnu.org "
-"containing the version (%s), and the following message:\n");
+"\n\nYou found a bug in GNU Parted! Here's what you have to do:\n\n"
+"Don't panic! The bug has most likely not affected any of your data.\n"
+"Help us to fix this bug by doing the following:\n\n"
+"Check whether the bug has already been fixed by checking\n"
+"the last version of GNU Parted that you can find at:\n\n"
+"\thttp://ftp.gnu.org/gnu/parted/\n\n"
+"Please check this version prior to bug reporting.\n\n"
+"If this has not been fixed yet or if you don't know how to check,\n"
+"please visit the GNU Parted website:\n\n"
+"\thttp://www.gnu.org/software/parted\n\n"
+"for further information.\n\n"
+"Your report should contain the version of this release (%s)\n"
+"along with the error message below, the output of\n\n"
+"\tparted DEVICE unit co print unit s print\n\n"
+"and additional information about your setup you consider important.\n");
 
 #define MAX_WORDS	1024
 
@@ -102,6 +109,13 @@ static struct {
 
 static PedExceptionOption	current_exception_opt = 0;
 
+static struct sigaction sig_segv;
+static struct sigaction sig_int;
+static struct sigaction sig_fpe;
+static struct sigaction sig_ill;
+
+volatile int got_ctrl_c = 0; /* used in exception_handler */
+
 int
 screen_width ()
 {
@@ -171,34 +185,185 @@ _add_history_unique (const char* line)
 }
 #endif /* HAVE_LIBREADLINE */
 
+/* Resets the environment by jumping to the initial state
+ * saved during ui intitialisation.
+ * Pass 1 as the parameter if you want to quit parted, 
+ * 0 if you just want to reset to the command prompt.
+ */
 static void
-interrupt_handler (int signum)
+reset_env (int quit)
 {
-	int	in_readline = readline_state.in_readline;
+    int	in_readline = readline_state.in_readline;
 
 	readline_state.in_readline = 0;
-
-	signal (signum, &interrupt_handler);
-
+	
 	if (in_readline) {
 		printf ("\n");
-		siglongjmp (readline_state.jmp_state, 1);
+        if (quit)    
+    	    exit (0);
+
+        siglongjmp (readline_state.jmp_state, 1);
 	}
 }
 
+#ifdef HAVE_SIGACTION
+
+/* Signal handler for SIGINT */
+static void
+sigint_handler (int signum, siginfo_t* info, void *ucontext)
+{
+    got_ctrl_c = 1;
+    sigaction (SIGINT, &sig_int, NULL);
+    reset_env (0);
+}
+
+/* Signal handler for SIGSEGV */
+static void
+sigsegv_handler (int signum, siginfo_t* info, void* ucontext)
+{   
+    sigaction (SIGINT, &sig_segv, NULL);
+    printf (bug_msg, VERSION);
+    
+    switch (info->si_code) {
+        
+        case SEGV_MAPERR:
+            printf(_("\nError: SEGV_MAPERR (Address not mapped "
+                     "to object)"));
+            break;
+
+        case SEGV_ACCERR:
+            printf(_("\nError: SEGV_ACCERR (Invalid permissions "
+                     "for mapped object)"));
+            break;
+
+        default:
+            printf(_("\nError: A general SIGSEGV signal was "
+                     "encountered."));
+            break;
+    }
+
+    abort();
+}
+
+/* Signal handler for SIGFPE */
+static void
+sigfpe_handler (int signum, siginfo_t* info, void* ucontext)
+{
+    sigaction (SIGFPE, &sig_fpe, NULL);
+    printf (bug_msg, VERSION);
+
+    switch (info->si_code) {
+
+        case FPE_INTDIV:
+            printf(_("\nError: FPE_INTDIV (Integer: divide by zero)"));
+            break;
+        
+        case FPE_INTOVF:
+            printf(_("\nError: FPE_INTOVF (Integer: overflow)"));
+            break;
+        
+        case FPE_FLTDIV:
+            printf(_("\nError: FPE_FLTDIV (Float: divide by zero)"));
+            break;
+
+        case FPE_FLTOVF:
+            printf(_("\nError: FPE_FLTOVF (Float: overflow)"));
+            break;
+
+        case FPE_FLTUND:
+            printf(_("\nError: FPE_FLTUND (Float: underflow)"));
+            break;
+
+        case FPE_FLTRES:
+            printf(_("\nError: FPE_FLTRES (Float: inexact result)"));
+            break;
+
+        case FPE_FLTINV:
+            printf(_("\nError: FPE_FLTINV (Float: invalid operation)"));
+            break;
+
+        case FPE_FLTSUB:
+            printf(_("\nError: FPE_FLTSUB (Float: subscript out of "
+                     "range)"));
+            break;
+
+        default:
+            printf(_("\nError: A general SIGFPE signal was "
+                     "encountered."));
+            break;
+
+    }
+    
+    abort();
+}
+
+/* Signal handler for SIGILL */
+static void
+sigill_handler (int signum, siginfo_t* info, void* ucontext)
+{
+    sigaction (SIGILL, &sig_ill, NULL);
+    printf (bug_msg, VERSION);
+
+    switch (info->si_code) {
+
+        case ILL_ILLOPC:
+            printf(_("\nError: ILL_ILLOPC (Illegal Opcode)"));
+            break;
+        
+        case ILL_ILLOPN:
+            printf(_("\nError: ILL_ILLOPN (Illegal Operand)"));
+            break;
+        
+        case ILL_ILLADR:
+            printf(_("\nError: ILL_ILLADR (Illegal addressing "
+                     "mode)"));
+            break;
+
+        case ILL_ILLTRP:
+            printf(_("\nError: ILL_ILLTRP (Illegal Trap)"));
+            break;
+
+        case ILL_PRVOPC:
+            printf(_("\nError: ILL_PRVOPC (Privileged Opcode)"));
+            break;
+
+        case ILL_PRVREG:
+            printf(_("\nError: ILL_PRVREG (Privileged Register)"));
+            break;
+
+        case ILL_COPROC:
+            printf(_("\nError: ILL_COPROC (Coprocessor Error)"));
+            break;
+
+        case ILL_BADSTK:
+            printf(_("\nError: ILL_BADSTK (Internal Stack Error)"));
+            break;
+
+        default:
+            printf(_("\nError: A general SIGILL signal was "
+                     "encountered."));
+            break;
+    }
+    
+    abort();
+}
+
+#endif /* HAVE_SIGACTION */
+
 static char*
 _readline (const char* prompt, const StrList* possibilities)
 {
-	char*		line;
+	char*	line;
+        int     val;
 
 	readline_state.possibilities = possibilities;
 	readline_state.cur_pos = NULL;
 	readline_state.in_readline = 1;
 
-	if (sigsetjmp (readline_state.jmp_state, 1))
-		return NULL;
+    if (sigsetjmp (readline_state.jmp_state,1))
+        return NULL;
 
-	wipe_line ();
+    wipe_line ();
 #ifdef HAVE_LIBREADLINE
 	if (!opt_script_mode) {
 		/* XXX: why isn't prompt const? */
@@ -250,7 +415,7 @@ _print_exception_text (PedException* ex)
 
 	if (ex->type == PED_EXCEPTION_BUG) {
 		printf (bug_msg, VERSION);
-		text = str_list_create ("\n", ex->message, "\n", NULL);
+		text = str_list_create ("\n", ex->message, "\n\n", NULL);
 	} else {
 		text = str_list_create (
 			   _(ped_exception_get_type_string (ex->type)),
@@ -277,9 +442,17 @@ exception_handler (PedException* ex)
 	if (opt_script_mode)
 		return PED_EXCEPTION_UNHANDLED;
 
+	got_ctrl_c = 0;
+
 	do {
 		opt = command_line_get_ex_opt ("", ex->options);
-	} while (opt == PED_EXCEPTION_UNHANDLED && isatty (0));
+	} while (opt == PED_EXCEPTION_UNHANDLED && isatty (0) && !got_ctrl_c);
+
+	if (got_ctrl_c) {
+                got_ctrl_c = 0;
+		opt = PED_EXCEPTION_UNHANDLED;
+	}
+
 	return opt;
 }
 
@@ -482,6 +655,14 @@ command_line_prompt_words (const char* prompt, const char* def,
 		free (_def);
 }
 
+/**
+ * Get a word from command line.
+ *
+ * \param possibilities a StrList of valid strings, NULL if all are valid.
+ * \param multi_word whether multiple words are allowed.
+ *
+ * \return The word(s), or NULL if empty.
+ */
 char*
 command_line_get_word (const char* prompt, const char* def,
 		       const StrList* possibilities, int multi_word)
@@ -508,55 +689,70 @@ command_line_get_word (const char* prompt, const char* def,
 }
 
 int
-command_line_get_integer (const char* prompt, int def, int* result)
+command_line_get_integer (const char* prompt, int* value)
 {
 	char	def_str [10];
 	char*	input;
 	int	valid;
 
-	snprintf (def_str, 10, "%d", def);
-	input = command_line_get_word (prompt, def ? def_str : NULL, NULL, 1);
+	snprintf (def_str, 10, "%d", *value);
+	input = command_line_get_word (prompt, *value ? def_str : NULL,
+				       NULL, 1);
 	if (!input)
 		return 0;
-	valid = sscanf (input, "%d", result);
-	free (input);
-	return valid;
-}
-
-PedSector
-command_line_get_sector (const char* prompt, PedDevice* dev,
-			 PedSector def, PedSector* result)
-{
-	char		def_str [16];
-	char*		input;
-	double		input_double;
-	int		valid;
-
-	snprintf (def_str, 16, "%.4f", (float) def / MEGABYTE_SECTORS);
-	input = command_line_get_word (prompt, def ? def_str : NULL, NULL, 1);
-	if (!input)
-		return 0;
-	valid = sscanf (input, "%lf", &input_double);
-	*result = (PedSector) (input_double * MEGABYTE_SECTORS);
-
-	/* negative numbers should count from end, and "-0" being the end */
-	if (input[0] == '-')
-		*result += dev->length;
-
-	/* don't be too pedantic about "outside disk" errors */
-	if (*result > dev->length - 1 && *result < dev->length * 1.01)
-		*result = dev->length - 1;
+	valid = sscanf (input, "%d", value);
 	free (input);
 	return valid;
 }
 
 int
-command_line_get_state (const char* prompt, int def, int* result)
+command_line_get_sector (const char* prompt, PedDevice* dev, PedSector* value,
+			 PedGeometry** range)
+{
+	char*	def_str;
+	char*	input;
+	int	valid;
+
+	def_str = ped_unit_format (dev, *value);
+	input = command_line_get_word (prompt, *value ? def_str : NULL,
+				       NULL, 1);
+
+	/* def_str might have rounded *value a little bit.  If the user picked
+	 * the default, make sure the selected sector is identical to the
+	 * default.
+	 */
+	if (input && *value && !strcmp (input, def_str)) {
+		if (range) {
+			*range = ped_geometry_new (dev, *value, 1);
+			ped_free (def_str);
+			return *range != NULL;
+		}
+
+		ped_free (def_str);
+		return 1;
+	}
+
+	ped_free (def_str);
+	if (!input) {
+		*value = 0;
+		if (range)
+			*range = NULL;
+		return 0;
+	}
+
+	valid = ped_unit_parse (input, dev, value, range);
+
+	free (input);
+	return valid;
+}
+
+int
+command_line_get_state (const char* prompt, int* value)
 {
 	char*	def_word;
 	char*	input;
 
-	if (def)
+	if (*value)
 		def_word = str_list_convert_node (on_list);
 	else
 		def_word = str_list_convert_node (off_list);
@@ -565,100 +761,132 @@ command_line_get_state (const char* prompt, int def, int* result)
 	if (!input)
 		return 0;
 	if (str_list_match_any (on_list, input))
-		*result = 1;
+		*value = 1;
 	else
-		*result = 0;
+		*value = 0;
 	free (input);
 	return 1;
 }
 
-PedDevice*
-command_line_get_device (const char* prompt, PedDevice* def)
+int
+command_line_get_device (const char* prompt, PedDevice** value)
 {
-	char*		def_dev_name = def ? def->path : NULL;
+	char*		def_dev_name = *value ? (*value)->path : NULL;
 	char*		dev_name;
 	PedDevice*	dev;
 
 	dev_name = command_line_get_word (prompt, def_dev_name, NULL, 1);
 	if (!dev_name)
-		return NULL;
+		return 0;
 	dev = ped_device_get (dev_name);
 	free (dev_name);
-	return dev;
-}
-
-PedDisk*
-command_line_get_disk (const char* prompt, PedDisk* def)
-{
-	PedDevice*	def_dev = def ? def->dev : NULL;
-	PedDevice*	dev = command_line_get_device (prompt, def_dev);
 	if (!dev)
-		return NULL;
-	if (dev == def->dev)
-		return def;
-	else
-		return ped_disk_new (dev);
+		return 0;
+	*value = dev;
+	return 1;
 }
 
-PedPartition*
-command_line_get_partition (const char* prompt, PedDisk* disk)
+int
+command_line_get_disk (const char* prompt, PedDisk** value)
 {
-	PedPartition*	part;
-	int		num;
-
-	if (!command_line_get_integer (prompt, 0, &num)) {
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-				     _("Expecting a partition number."));
-		return NULL;
+	PedDevice*	dev = *value ? (*value)->dev : NULL;
+	if (!command_line_get_device (prompt, &dev))
+		return 0;
+	if (dev != (*value)->dev) {
+		PedDisk* new_disk = ped_disk_new (dev);
+		if (!new_disk)
+			return 0;
+		*value = new_disk;
 	}
-
-	part = ped_disk_get_partition (disk, num);
-	if (!part)
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-				     _("Partition doesn't exist."));
-	return part;
+	return 1;
 }
 
-const PedFileSystemType*
-command_line_get_fs_type (const char* prompt, const PedFileSystemType* def)
+int
+command_line_get_partition (const char* prompt, PedDisk* disk,
+			    PedPartition** value)
+{
+    PedPartition*   part;
+    
+    /* Flawed logic, doesn't seem to work?! 
+    check = ped_disk_next_partition (disk, part);
+    part  = ped_disk_next_partition (disk, check);
+    
+    if (part == NULL) {
+    
+        *value = check; 	 
+        printf (_("The (only) primary partition has "
+                  "been automatically selected\n"));
+        return 1; 	 
+
+    } else {
+    */
+        int num = (*value) ? (*value)->num : 0;
+
+        if (!command_line_get_integer (prompt, &num)) {
+            ped_exception_throw (PED_EXCEPTION_ERROR,
+                                 PED_EXCEPTION_CANCEL,
+                                 _("Expecting a partition number."));
+            return 0;
+        }
+
+        part = ped_disk_get_partition (disk, num);
+
+        if (!part) {
+                ped_exception_throw (PED_EXCEPTION_ERROR,
+                                     PED_EXCEPTION_CANCEL,
+                                     _("Partition doesn't exist."));
+            return 0;
+        }
+
+        *value = part;
+        return 1;
+    //}
+}
+
+int
+command_line_get_fs_type (const char* prompt, const PedFileSystemType*(* value))
 {
 	char*			fs_type_name;
 	PedFileSystemType*	fs_type;
 
-	fs_type_name = command_line_get_word (prompt, def ? def->name : NULL,
+	fs_type_name = command_line_get_word (prompt,
+					      *value ? (*value)->name : NULL,
 		       			      fs_type_list, 1);
 	if (!fs_type_name) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 				     _("Expecting a file system type."));
-		return NULL;
+		return 0;
 	}
 	fs_type = ped_file_system_type_get (fs_type_name);
 	if (!fs_type) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 				     _("Unknown file system type \"%s\"."),
 				     fs_type_name);
-		return NULL;
+		return 0;
 	}
 	free (fs_type_name);
-	return fs_type;
+	*value = fs_type;
+	return 1;
 }
 
-const PedDiskType*
-command_line_get_disk_type (const char* prompt, const PedDiskType* def)
+int
+command_line_get_disk_type (const char* prompt, const PedDiskType*(* value))
 {
 	char*		disk_type_name;
 	PedDiskType*	disk_type;
 
-	disk_type_name = command_line_get_word (prompt, def ? def->name : NULL,
+	disk_type_name = command_line_get_word (prompt,
+						*value ? (*value)->name : NULL,
 						disk_type_list, 1);
 	if (!disk_type_name) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 				     _("Expecting a disk label type."));
-		return NULL;
+		return 0;
 	}
-	disk_type = ped_disk_type_get (disk_type_name);
+	*value = ped_disk_type_get (disk_type_name);
 	free (disk_type_name);
-	return disk_type;
+	PED_ASSERT (*value != NULL, return 0);
+	return 1;
 }
 
 int
@@ -684,7 +912,7 @@ command_line_get_part_flag (const char* prompt, const PedPartition* part,
 
 	if (flag_name) {
 		*flag = ped_partition_flag_get_by_name (flag_name);
-		free (flag_name);
+		ped_free (flag_name);
 		return 1;
 	} else {
 		return 0;
@@ -725,8 +953,8 @@ _can_create_logical (const PedDisk* disk)
 }
 
 int
-command_line_get_new_part_type (const char* prompt, const PedDisk* disk,
-	       			PedPartitionType* type)
+command_line_get_part_type (const char* prompt, const PedDisk* disk,
+	       		    PedPartitionType* type)
 {
 	StrList*	opts = NULL;
 	char*		type_name;
@@ -811,6 +1039,30 @@ command_line_get_ex_opt (const char* prompt, PedExceptionOption options)
 	return opt;
 }
 
+int
+command_line_get_unit (const char* prompt, PedUnit* unit)
+{
+	StrList*	opts = NULL;
+	PedUnit		walk;
+	char*		unit_name;
+	const char*	default_unit_name;
+
+	for (walk = PED_UNIT_FIRST; walk <= PED_UNIT_LAST; walk++)
+		opts = str_list_append (opts, ped_unit_get_name (walk));
+
+	default_unit_name = ped_unit_get_name (ped_unit_get_default ());
+	unit_name = command_line_get_word (prompt, default_unit_name, opts, 1);
+	str_list_destroy (opts);
+
+	if (unit_name) {
+		*unit = ped_unit_get_by_name (unit_name);
+		free (unit_name);
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
 int
 command_line_is_integer ()
 {
@@ -929,9 +1181,32 @@ init_ui ()
 	readline_state.in_readline = 0;
 #endif
 
-	signal (SIGINT, &interrupt_handler);
+#ifdef HAVE_SIGACTION
+    sigset_t curr;
+    sigfillset (&curr);
 
-	return 1;
+    sig_segv.sa_sigaction = &sigsegv_handler;
+    sig_int.sa_sigaction  = &sigint_handler;
+    sig_fpe.sa_sigaction  = &sigfpe_handler;
+    sig_ill.sa_sigaction  = &sigill_handler;
+
+    sig_segv.sa_mask = 
+        sig_int.sa_mask = 
+            sig_fpe.sa_mask = 
+                sig_ill.sa_mask = curr;
+    
+    sig_segv.sa_flags = 
+        sig_int.sa_flags = 
+            sig_fpe.sa_flags = 
+                sig_ill.sa_flags = SA_SIGINFO;
+
+    sigaction (SIGSEGV, &sig_segv, NULL);
+    sigaction (SIGINT, &sig_int, NULL);
+    sigaction (SIGFPE, &sig_fpe, NULL);
+    sigaction (SIGILL, &sig_ill, NULL);
+#endif /* HAVE_SIGACTION */
+
+    return 1;
 }
 
 void
@@ -974,12 +1249,12 @@ interactive_mode (PedDevice** dev, Command* cmd_list[])
 
 	printf ("%s", prog_name);
 
-	list = str_list_create (_(banner_msg), NULL);
+    print_using_dev (*dev);
+
+    list = str_list_create (_(banner_msg), NULL);
 	str_list_print_wrap (list, screen_width (), 0, 0);
 	str_list_destroy (list);
 
-	print_using_dev (*dev);
-
 	while (1) {
 		char*		word;
 		Command*	cmd;
@@ -1033,6 +1308,14 @@ non_interactive_mode (PedDevice** dev, Command* cmd_list[],
 			help_msg ();
 			goto error;
 		}
+
+                if (!(cmd->non_interactive)) {
+                        printf("This command does not make sense in "
+                               "non-interactive mode.\n");
+                        exit(1);
+                        goto error;
+                }
+                
 		if (!command_run (cmd, dev))
 			goto error;
 	}
